#include <errno.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "log.h"
#include "util.h"

using namespace std;

int make_tempfile(const std::string &contents)
{
	char filename[] = "/tmp/cubemap.XXXXXX";
	mode_t old_umask = umask(0600);
	int fd = mkstemp(filename);
	umask(old_umask);
	if (fd == -1) {
		log_perror("mkstemp");
		return -1;
	}

	if (unlink(filename) == -1) {
		log_perror("unlink");
		// Can still continue;
	}

	const char *ptr = contents.data();
	size_t to_write = contents.size();
	while (to_write > 0) {
		ssize_t ret = write(fd, ptr, to_write);
		if (ret == -1) {
			log_perror("write");
			close(fd);
			return -1;
		}

		ptr += ret;
		to_write -= ret;
	}

	return fd;
}

bool read_tempfile_and_close(int fd, std::string *contents)
{
	bool ok = read_tempfile(fd, contents);

	int ret;
	do {
		ret = close(fd);  // Implicitly deletes the file.
	} while (ret == -1 && errno == EINTR);
	
	if (ret == -1) {
		log_perror("close");
		// Can still continue.
	}

	return ok;
}

bool read_tempfile(int fd, std::string *contents)
{
	ssize_t ret, has_read;

	off_t len = lseek(fd, 0, SEEK_END);
	if (len == -1) {
		log_perror("lseek");
		return false;
	}

	contents->resize(len);

	if (lseek(fd, 0, SEEK_SET) == -1) {
		log_perror("lseek");
		return false;
	}

	has_read = 0;
	while (has_read < len) {
		ret = read(fd, &((*contents)[has_read]), len - has_read);
		if (ret == -1) {
			log_perror("read");
			return false;
		}
		if (ret == 0) {
			log(ERROR, "Unexpected EOF!");
			return false;
		}
		has_read += ret;
	}

	return true;
}
